@@ -43,6 +43,9 @@ from agentle.agents.execution_state import ExecutionState
 from agentle.agents.step import Step
 from agentle.generations.models.generation.usage import Usage
 from agentle.generations.models.message_parts.text import TextPart
+from agentle.generations.models.message_parts.tool_execution_suggestion import (
+    ToolExecutionSuggestion,
+)
 from agentle.generations.models.messages.assistant_message import AssistantMessage
 from agentle.generations.models.messages.developer_message import DeveloperMessage
 from agentle.generations.models.messages.user_message import UserMessage
@@ -148,6 +151,41 @@ class Context(BaseModel):
     tags: MutableSequence[str] = Field(default_factory=list)
     """Optional tags for categorizing or filtering contexts."""
 
+    def replace_message_history(
+        self,
+        new_history: Sequence[DeveloperMessage | UserMessage | AssistantMessage],
+        keep_developer_messages: bool = False,
+    ) -> None:
+        """
+        Replace the message history with a new sequence of messages.
+
+        Args:
+            new_history: The new sequence of messages to set as the context's message history
+            reset_execution_state: Whether to reset the execution state to its initial values
+            keep_developer_messages: Whether to keep existing developer messages in the history
+        """
+        if keep_developer_messages:
+            new_history = [
+                m for m in new_history if not isinstance(m, DeveloperMessage)
+            ] + list(self.message_history)
+            self.message_history = new_history
+            return
+
+        self.message_history = list(new_history)
+        self.execution_state.last_updated_at = datetime.now()
+
+    @property
+    def last_message(self) -> DeveloperMessage | AssistantMessage | UserMessage:
+        return self.message_history[-1]
+
+    @property
+    def tool_execution_suggestions(self) -> Sequence[ToolExecutionSuggestion]:
+        suggestions: MutableSequence[ToolExecutionSuggestion] = []
+        for step in self.steps:
+            suggestions.extend(step.tool_execution_suggestions)
+
+        return suggestions
+
     @property
     def tool_execution_results(self) -> Sequence[ToolExecutionResult]:
         results: MutableSequence[ToolExecutionResult] = []
@@ -156,6 +194,12 @@ class Context(BaseModel):
 
         return results
 
+    def add_messages(
+        self, messages: Sequence[DeveloperMessage | UserMessage | AssistantMessage]
+    ) -> None:
+        self.message_history.extend(messages)
+        self.execution_state.last_updated_at = datetime.now()
+
     def add_user_message(self, text: str) -> None:
         """
         Add a user message to the conversation history.
