@@ -2,7 +2,7 @@ from __future__ import annotations
 
 import asyncio
 from collections.abc import Mapping
-from typing import Any, Literal, Sequence, override
+from typing import TYPE_CHECKING, Any, Literal, Sequence, override
 
 import httpx
 
@@ -27,10 +27,12 @@ from agentle.generations.providers.openai.adapters.chat_completion_to_generation
 )
 from agentle.generations.providers.types.model_kind import ModelKind
 from agentle.generations.tools.tool import Tool
-from agentle.generations.tracing.contracts.stateful_observability_client import (
-    StatefulObservabilityClient,
-)
-from agentle.generations.tracing.decorators.observe import observe
+from agentle.generations.tracing import observe
+
+if TYPE_CHECKING:
+    from openai import AsyncOpenAI
+    from agentle.generations.tracing.otel_client import OtelClient
+
 
 type WithoutStructuredOutput = None
 
@@ -48,11 +50,13 @@ class OpenaiGenerationProvider(GenerationProvider):
     OpenAI generation provider.
     """
 
+    client: AsyncOpenAI
+
     def __init__(
         self,
-        api_key: str,
+        api_key: str | None = None,
         *,
-        tracing_client: StatefulObservabilityClient | None = None,
+        otel_clients: Sequence[OtelClient] | OtelClient | None = None,
         organization_name: str | None = None,
         project_name: str | None = None,
         base_url: str | httpx.URL | None = None,
@@ -64,7 +68,7 @@ class OpenaiGenerationProvider(GenerationProvider):
     ) -> None:
         from openai import AsyncOpenAI
 
-        super().__init__(tracing_client=tracing_client)
+        super().__init__(otel_clients=otel_clients)
 
         self._client = AsyncOpenAI(
             api_key=api_key,
@@ -114,6 +118,7 @@ class OpenaiGenerationProvider(GenerationProvider):
         """
         from openai._types import NOT_GIVEN as OPENAI_NOT_GIVEN
         from openai.types.chat.chat_completion import ChatCompletion
+        from openai.types.chat.parsed_chat_completion import ParsedChatCompletion
 
         _generation_config = self._normalize_generation_config(generation_config)
 
@@ -122,7 +127,7 @@ class OpenaiGenerationProvider(GenerationProvider):
 
         try:
             async with asyncio.timeout(_generation_config.timeout_in_seconds):
-                chat_completion: ChatCompletion = (
+                chat_completion: ChatCompletion | ParsedChatCompletion[T] = (
                     await self._client.chat.completions.create(
                         model=self._resolve_model(model),
                         messages=[
@@ -132,6 +137,17 @@ class OpenaiGenerationProvider(GenerationProvider):
                         if tools
                         else OPENAI_NOT_GIVEN,
                     )
+                    if not bool(response_schema)
+                    else await self._client.chat.completions.parse(
+                        model=self._resolve_model(model),
+                        messages=[
+                            input_message_adapter.adapt(message) for message in messages
+                        ],
+                        tools=[openai_tool_adapter.adapt(tool) for tool in tools]
+                        if tools
+                        else OPENAI_NOT_GIVEN,
+                        response_format=response_schema,
+                    )
                 )
         except asyncio.TimeoutError as e:
             e.add_note(
@@ -139,10 +155,7 @@ class OpenaiGenerationProvider(GenerationProvider):
             )
             raise
 
-        output_adapter = ChatCompletionToGenerationAdapter[T](
-            response_schema=response_schema
-        )
-
+        output_adapter = ChatCompletionToGenerationAdapter[T]()
         return output_adapter.adapt(chat_completion)
 
     @property
