@@ -3,14 +3,16 @@ MCP Servers Package
 
 This package provides server implementations for the Model Context Protocol.
 It includes protocol definitions and concrete implementations for different
-server types (e.g., HTTP, WebSocket) that allow communication with external
+server types (e.g., HTTP, WebSocket, SSE) that allow communication with external
 tools and resources.
 """
 
-from agentle.mcp.servers.http_mcp_server import HTTPMCPServer
+from agentle.mcp.servers.streamable_http_mcp_server import StreamableHTTPMCPServer
+from agentle.mcp.servers.sse_mcp_server import SSEMCPServer
 from agentle.mcp.servers.mcp_server_protocol import MCPServerProtocol
 
 __all__ = [
-    "HTTPMCPServer",
+    "StreamableHTTPMCPServer",
+    "SSEMCPServer",
     "MCPServerProtocol",
 ]
