@@ -8,28 +8,22 @@ organize the document content.
 
 import hashlib
 from pathlib import Path
-from typing import Literal, override
+from typing import Literal
 
 from rsb.functions.ext2mime import ext2mime
 from rsb.models.field import Field
 
-from agentle.agents.agent import Agent
+
 from agentle.generations.models.message_parts.file import FilePart
 from agentle.generations.models.structured_outputs_store.visual_media_description import (
     VisualMediaDescription,
 )
-from agentle.generations.providers.base.generation_provider import GenerationProvider
 
-from agentle.generations.providers.google.google_generation_provider import (
-    GoogleGenerationProvider,
-)
-from agentle.parsing.document_parser import DocumentParser
-from agentle.parsing.factories.visual_description_agent_default_factory import (
-    visual_description_agent_default_factory,
-)
+from agentle.generations.providers.base.generation_provider import GenerationProvider
 from agentle.parsing.image import Image
 from agentle.parsing.parsed_file import ParsedFile
 from agentle.parsing.section_content import SectionContent
+from agentle.parsing.document_parser import DocumentParser
 
 
 class DocxFileParser(DocumentParser):
@@ -76,7 +70,7 @@ class DocxFileParser(DocumentParser):
     *   `multi_modal_provider` (GenerationProvider):
         An alternative to using a visual_description_agent. This is a generation
         provider capable of handling multi-modal content (text and images).
-        Defaults to GoogleGenaiGenerationProvider().
+        Defaults to GoogleGenerationProvider().
 
         Note: You cannot use both visual_description_agent and multi_modal_provider
         at the same time.
@@ -121,23 +115,14 @@ class DocxFileParser(DocumentParser):
 
     strategy: Literal["high", "low"] = Field(default="high")
 
-    visual_description_agent: Agent[VisualMediaDescription] = Field(
-        default_factory=visual_description_agent_default_factory,
+    visual_description_provider: GenerationProvider | None = Field(
+        default=None,
     )
     """
     The agent to use for generating the visual description of the document.
     Useful when you want to customize the prompt for the visual description.
     """
 
-    multi_modal_provider: GenerationProvider = Field(
-        default_factory=GoogleGenerationProvider,
-    )
-    """
-    The multi-modal provider to use for generating the visual description of the document.
-    Useful when you want us to customize the prompt for the visual description.
-    """
-
-    @override
     async def parse_async(
         self,
         document_path: str,
@@ -202,7 +187,8 @@ class DocxFileParser(DocumentParser):
 
         final_images: list[Image] = []
         image_descriptions: list[str] = []
-        if self.visual_description_agent and self.strategy == "high":
+
+        if self.visual_description_provider and self.strategy == "high":
             for idx, (image_name, image_bytes) in enumerate(doc_images, start=1):
                 image_hash = hashlib.sha256(image_bytes).hexdigest()
 
@@ -215,8 +201,10 @@ class DocxFileParser(DocumentParser):
                         mime_type=ext2mime(extension),
                         data=image_bytes,
                     )
-                    agent_response = await self.visual_description_agent.run_async(
-                        agent_input
+                    agent_response = await self.visual_description_provider.generate_by_prompt_async(
+                        agent_input,
+                        developer_prompt="You are a helpful assistant that deeply understands visual media.",
+                        response_schema=VisualMediaDescription,
                     )
                     image_md = agent_response.parsed.md
                     ocr_text = agent_response.parsed.ocr_text or ""
